"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.loadDataSet = exports.newDataSet = exports.newColumnDescriptions = exports.LstDataAttachment = exports.LstData = void 0;
/* eslint-disable @typescript-eslint/no-explicit-any */
require("reflect-metadata");
const ColumnDescription_1 = require("./ColumnDescription");
const ColumnDescriptions_1 = require("./ColumnDescriptions");
const DataSet_1 = require("./DataSet");
const DataSetValues_1 = require("./DataSetValues");
const lstDataMap = new WeakMap();
const dataSetClassByUnit = {
    mm: DataSetValues_1.DataSetMillimeter,
    kg: DataSetValues_1.DataSetKilogram,
    min: DataSetValues_1.DataSetMinutes,
    s: DataSetValues_1.DataSetSeconds,
    'kg/dm3': DataSetValues_1.DataSetKilogramPerLiter,
    'm/min': DataSetValues_1.DataSetMetersPerMinute,
    'm/s': DataSetValues_1.DataSetMetersPerSecond,
    W: DataSetValues_1.DataSetWatt,
    'm/s2': DataSetValues_1.DataSetMetersPerSquareSeconds,
    Hz: DataSetValues_1.DataSetHertz,
    bar: DataSetValues_1.DataSetBar,
    mm2: DataSetValues_1.DataSetSquareMillimeter,
    deg: DataSetValues_1.DataSetDegrees,
};
function getDefaultValue(type, options) {
    switch (type) {
        case Number:
            return options?.default ?? 0;
        case String:
            return options?.default ?? '';
        case Boolean:
            return options?.default ?? false;
        default:
            return options?.default;
    }
}
function getProxyFactory(type, propertyName, id, options) {
    if (options && options.read && options.write) {
        // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
        return (dataSet) => new DataSetValues_1.DataSetAny(dataSet, id, getDefaultValue(type, options), options.read, options.write);
    }
    else if (options?.read || options?.write) {
        throw new Error(`You need to specify both, read and write in options to @LstData for ${propertyName}`);
    }
    switch (type) {
        case Number:
            // eslint-disable-next-line no-case-declarations
            const dataSetClass = options?.unit ? dataSetClassByUnit[options.unit] ?? DataSetValues_1.DataSetNumber : DataSetValues_1.DataSetNumber;
            return (dataSet) => new dataSetClass(dataSet, id, getDefaultValue(type, options));
        case String:
            return (dataSet) => new DataSetValues_1.DataSetString(dataSet, id, getDefaultValue(type, options));
        case Boolean:
            return (dataSet) => new DataSetValues_1.DataSetBool(dataSet, id, getDefaultValue(type, options));
        default:
            throw new Error(`Unable to apply @LstData for ${propertyName} to type ${type}. Use read and write options.`);
    }
}
function getColumnDescriptionFactory(type, propertyName, id, options) {
    switch (type) {
        case Number:
        case Boolean:
            return () => ({ id, name: options?.name ?? propertyName, unit: options?.unit ?? '', valueType: ColumnDescription_1.ValueType.NUMBER });
        default:
            return () => ({ id, name: options?.name ?? propertyName, unit: options?.unit ?? '', valueType: ColumnDescription_1.ValueType.TEXT });
    }
}
function LstData(id, options) {
    // eslint-disable-next-line @typescript-eslint/explicit-module-boundary-types
    return function (target, propertyName) {
        const type = Reflect.getMetadata('design:type', target, propertyName);
        const proxyFactory = getProxyFactory(type, propertyName, id, options);
        const columnDescriptionFactory = getColumnDescriptionFactory(type, propertyName, id, options);
        const map = lstDataMap.get(target);
        if (map) {
            // eslint-disable-next-line security/detect-object-injection
            map[propertyName] = { proxyFactory, columnDescriptionFactory };
        }
        else {
            lstDataMap.set(target, { [propertyName]: { proxyFactory, columnDescriptionFactory } });
        }
    };
}
exports.LstData = LstData;
const lstDataAttachmentMap = new WeakMap();
function LstDataAttachment(options) {
    // eslint-disable-next-line @typescript-eslint/explicit-module-boundary-types
    return function (target, propertyName) {
        const type = Reflect.getMetadata('design:type', target, propertyName);
        if (type !== String && type !== Array && !(options?.read && options?.write)) {
            throw new Error(`@LstAttachment for ${propertyName} must be of type 'string[]' but found ${type}. Please define read and write in the options.`);
        }
        const map = lstDataAttachmentMap.get(target);
        if (map) {
            // eslint-disable-next-line security/detect-object-injection
            map[propertyName] = (options ?? {});
        }
        else {
            lstDataAttachmentMap.set(target, {
                [propertyName]: (options ?? {}),
            });
        }
    };
}
exports.LstDataAttachment = LstDataAttachment;
function newColumnDescriptions(factory) {
    const map = lstDataMap.get(factory.prototype);
    if (!map) {
        throw new Error(`Type ${factory} is not decorated with @LstData decorator.`);
    }
    return new ColumnDescriptions_1.ColumnDescriptions(Object.values(map).map((lstData) => lstData.columnDescriptionFactory()));
}
exports.newColumnDescriptions = newColumnDescriptions;
function newDataSet(factory, dataSet, ...args) {
    const map = lstDataMap.get(factory.prototype);
    const attachmentMap = lstDataAttachmentMap.get(factory.prototype);
    if (!map && !attachmentMap) {
        throw new Error(`Type ${factory} is not decorated with @LstData not @LstDataAttachment decorator.`);
    }
    const $dataSet = dataSet ?? new DataSet_1.DataSet(newColumnDescriptions(factory));
    const instance = new factory(...args);
    instance.$dataSet = $dataSet;
    if (map) {
        Object.entries(map).forEach(([propertyName, lstData]) => {
            const proxy = lstData.proxyFactory($dataSet);
            if (!dataSet) {
                // eslint-disable-next-line no-self-assign
                proxy.value = proxy.value; // Initialize in dataSet
            }
            const getter = () => proxy.value;
            const setter = (newVal) => (proxy.value = newVal);
            // eslint-disable-next-line security/detect-object-injection
            delete instance[propertyName];
            Object.defineProperty(instance, propertyName, { get: getter, set: setter, enumerable: true, configurable: true });
        });
    }
    if (attachmentMap) {
        Object.entries(attachmentMap).forEach(([propertyName, attachmentDef]) => {
            const getter = attachmentDef.read ? () => attachmentDef.read($dataSet.attachment) : () => $dataSet.attachment;
            const setter = attachmentDef.write ? (v) => ($dataSet.attachment = attachmentDef.write(v)) : (v) => ($dataSet.attachment = v);
            // eslint-disable-next-line security/detect-object-injection
            delete instance[propertyName];
            Object.defineProperty(instance, propertyName, { get: getter, set: setter, enumerable: true, configurable: true });
        });
    }
    return instance;
}
exports.newDataSet = newDataSet;
function loadDataSet(factory, dataSet, ...args) {
    const instance = newDataSet(factory, dataSet, ...args);
    // eslint-disable-next-line @typescript-eslint/no-unused-vars
    return (({ $dataSet, ...clone }) => clone)(instance);
}
exports.loadDataSet = loadDataSet;
//# sourceMappingURL=LstData.js.map