"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Reader = void 0;
/* eslint-disable @typescript-eslint/no-non-null-assertion */
const Attachment_1 = require("../model/Attachment");
const Document_1 = require("../model/Document");
const Constants_1 = require("./Constants");
const Parser_1 = require("./Parser");
const TableReader_1 = require("./TableReader");
class Reader {
    parser;
    document;
    documents = [];
    constructor(source) {
        this.parser = new Parser_1.Parser(source);
    }
    read() {
        this.documents = [];
        while (this.tryRead()) {
            if (this.document) {
                this.documents.push(this.document);
            }
        }
        return this;
    }
    tryRead() {
        this.document = undefined;
        let preamble = undefined;
        this.parser.tryReadContentLine();
        if (this.parser.token === Constants_1.Constants.TC500_BEGIN_DOCUMENT) {
            // if program is for a TC500 it might start with a % and the line should be skipped
            this.parser.tryReadContentLine();
            preamble = Constants_1.Constants.TC500_BEGIN_DOCUMENT;
        }
        const success = this.parser.token == Constants_1.Constants.BEGIN_DOCUMENT;
        if (success) {
            this.parser.tryReadContentLine();
            this.document = new Document_1.Document();
            if (preamble)
                this.document.preamble = preamble;
            this.readHeader();
            this.readTablesAndAttachments();
            if (this.parser.token != Constants_1.Constants.END_DOCUMENT) {
                throw new Error(`Expected end of document 'ED' in LST line ${this.parser.lineNumber}.`);
            }
        }
        return success;
    }
    readHeader() {
        while (this.parser.token != null && !this.parser.token.startsWith(Constants_1.Constants.Table.BEGIN_PREFIX)) {
            if (this.parser.token == Constants_1.Constants.Header.METRIC_MEASURE) {
                this.document.measuringSystem = Document_1.MeasuringSystem.METRIC;
            }
            else if (this.parser.token == Constants_1.Constants.Header.IMPERIAL_MEASURE) {
                this.document.measuringSystem = Document_1.MeasuringSystem.IMPERIAL;
            }
            else {
                throw new Error(`Unknown header line in LST line ${this.parser.lineNumber}.`);
            }
            this.parser.tryReadContentLine();
        }
    }
    readTablesAndAttachments() {
        this.document.tables = [];
        while (this.parser.token != null) {
            if (this.parser.token.startsWith(Constants_1.Constants.Table.BEGIN_PREFIX)) {
                this.readTable();
            }
            else if (this.parser.token.startsWith(Constants_1.Constants.Table.ATTACHMENT_START_PREFIX)) {
                this.readAttachment();
            }
            else {
                break;
            }
        }
    }
    readTable() {
        const tableReader = new TableReader_1.TableReader(this.parser);
        tableReader.read();
        this.document.tables.push(tableReader.table);
    }
    readAttachment() {
        const attachment = new Attachment_1.Attachment(this.parser.token.substring(Constants_1.Constants.Table.ATTACHMENT_START_PREFIX.length).trimEnd());
        const stopToken = `${Constants_1.Constants.Table.ATTACHMENT_STOP_PREFIX}${attachment.name}`;
        while (this.parser.tryReadLine() && !this.parser.token.startsWith(stopToken)) {
            attachment.data.push(this.parser.token);
        }
        this.parser.tryReadContentLine();
        this.document.attachments.push(attachment);
    }
}
exports.Reader = Reader;
//# sourceMappingURL=Reader.js.map