"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.TableReader = void 0;
/* eslint-disable @typescript-eslint/no-non-null-assertion */
const ColumnDescription_1 = require("../model/ColumnDescription");
const Table_1 = require("../model/Table");
const Constants_1 = require("./Constants");
const TableLineReader_1 = require("./TableLineReader");
class TableReader {
    parser;
    tableLineReader;
    table;
    constructor(parser) {
        this.parser = parser;
        this.tableLineReader = new TableLineReader_1.TableLineReader(parser);
    }
    read() {
        this.table = new Table_1.Table();
        this.readBegin();
        this.readColumnDescriptions();
        this.readData();
        this.readEnd();
    }
    readBegin() {
        if (this.parser.token === null || !this.parser.token.startsWith(Constants_1.Constants.Table.BEGIN_PREFIX)) {
            throw new Error(`Expected table definition in LST line ${this.parser.lineNumber}.`);
        }
        this.table.name = this.parser.token.substring(Constants_1.Constants.Table.BEGIN_PREFIX.length);
        this.tableLineReader.read();
    }
    readColumnDescriptions() {
        this.readCountDataSet(Constants_1.Constants.Table.ColumnDescription.ID);
        while (this.parser.isOk) {
            this.tableLineReader.read();
            if (!this.isColumDescriptionLine()) {
                break;
            }
            this.addColumnSpecification();
        }
    }
    addColumnSpecification() {
        if (this.tableLineReader.fields.length !== Constants_1.Constants.Table.ColumnDescription.FIELD_COUNT || !this.isColumDescriptionLine()) {
            throw new Error(`Invalid column definition in table ${this.table.name} in LST line ${this.parser.lineNumber}.`);
        }
        const id = +this.tableLineReader.fields[Constants_1.Constants.Table.ColumnDescription.COLUMN_ID_INDEX];
        const name = this.tableLineReader.fields[Constants_1.Constants.Table.ColumnDescription.NAME_INDEX];
        const unit = this.tableLineReader.fields[Constants_1.Constants.Table.ColumnDescription.UNIT_INDEX];
        const valueType = this.toValueType(this.tableLineReader.fields[Constants_1.Constants.Table.ColumnDescription.VALUE_TYPE_INDEX]);
        this.table.addColumn(id, name, unit, valueType);
    }
    toValueType(valueTypeString) {
        const valueType = {
            [Constants_1.Constants.Table.ColumnDescription.TEXT_VALUE_TYPE]: ColumnDescription_1.ValueType.TEXT,
            [Constants_1.Constants.Table.ColumnDescription.NUMBER_VALUE_TYPE]: ColumnDescription_1.ValueType.NUMBER,
            [Constants_1.Constants.Table.ColumnDescription.ENUM_VALUE_TYPE]: ColumnDescription_1.ValueType.ENUM,
            [Constants_1.Constants.Table.ColumnDescription.BOOLEAN_VALUE_TYPE]: ColumnDescription_1.ValueType.BOOLEAN,
        }[valueTypeString.toUpperCase()];
        if (valueType)
            return valueType;
        throw new Error(`Unknown value type ${valueTypeString} in LST line ${this.parser.lineNumber}.`);
    }
    readData() {
        this.readCountDataSet(Constants_1.Constants.Table.Data.ID);
        while (this.parser.isOk) {
            this.tableLineReader.read();
            if (this.isDataSetLine()) {
                this.addDataSet();
            }
            else if (this.isAttachmentStartLine()) {
                const attachment = this.readAttachment();
                this.addAttachment(attachment);
            }
            else if (this.isLineToIgnore()) {
                // Ignore this line.
            }
            else {
                break;
            }
        }
    }
    addDataSet() {
        if (this.tableLineReader.fields.length !== this.table.columnDescriptions.length + 1 || // +1 due to 'DA'.
            !this.isDataSetLine()) {
            throw new Error(`Invalid data set in table ${this.table.name} in LST line ${this.parser.lineNumber}.`);
        }
        const dataSet = this.table.addDataSet();
        dataSet.values = this.tableLineReader.fields.slice(1); // Remove 'DA' field.
        if (this.parser.isOk && this.parser.token === Constants_1.Constants.Table.Data.BEGIN_ATTACHMENT) {
            dataSet.attachment = this.readAttachment();
        }
    }
    readAttachment() {
        const content = [];
        while (this.parser.tryReadLine() && this.parser.token !== Constants_1.Constants.Table.Data.END_ATTACHMENT) {
            content.push(this.parser.token);
        }
        return content;
    }
    addAttachment(attachment) {
        if (this.table.dataSets.length === 0) {
            throw new Error(`Text attachment without data set LST line ${this.parser.lineNumber}.`);
        }
        this.table.dataSets[this.table.dataSets.length - 1].attachment = attachment;
    }
    readEnd() {
        if (this.tableLineReader.fields.length !== 1 || this.tableLineReader.fields[0] !== Constants_1.Constants.Table.END_PREFIX + this.table.name) {
            throw new Error(`Missing end of table ${this.table.name} in LST line ${this.parser.lineNumber}.`);
        }
        this.parser.tryReadContentLine();
    }
    readCountDataSet(id) {
        if (this.tableLineReader.fields.length !== Constants_1.Constants.Table.Count.FIELDS_COUNT ||
            this.tableLineReader.fields[Constants_1.Constants.Table.Count.ID_INDEX] !== Constants_1.Constants.Table.Count.ID ||
            this.tableLineReader.fields[Constants_1.Constants.Table.Count.TARGET_ID_INDEX] !== id) {
            throw new Error(`Missing count of data sets in table definition in LST line ${this.parser.lineNumber}.`);
        }
        const count = +this.tableLineReader.fields[Constants_1.Constants.Table.Count.COUNT_INDEX];
        return count;
    }
    isColumDescriptionLine() {
        const fields = this.tableLineReader.fields;
        return fields.length > 1 && fields[Constants_1.Constants.Table.ColumnDescription.ID_INDEX].toUpperCase() === Constants_1.Constants.Table.ColumnDescription.ID;
    }
    isDataSetLine() {
        const fields = this.tableLineReader.fields;
        return fields.length > 1 && fields[Constants_1.Constants.Table.Data.ID_INDEX].toUpperCase() === Constants_1.Constants.Table.Data.ID;
    }
    isAttachmentStartLine() {
        return this.tableLineReader.fields.length === 1 && this.tableLineReader.fields[0] === Constants_1.Constants.Table.Data.BEGIN_ATTACHMENT;
    }
    isLineToIgnore() {
        return this.tableLineReader.fields.length >= 1 && this.tableLineReader.fields[Constants_1.Constants.Table.Data.ID_INDEX] === Constants_1.Constants.Table.Data.BNC_GE;
    }
}
exports.TableReader = TableReader;
//# sourceMappingURL=TableReader.js.map