"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Writer = void 0;
const ColumnDescription_1 = require("../model/ColumnDescription");
const Document_1 = require("../model/Document");
const Constants_1 = require("./Constants");
class Writer {
    NEWLINE = '\r\n';
    MAX_LINE_LENGTH = 71;
    writeDocuments(documents) {
        return documents.map((x) => this.writeDocument(x)).join('C' + this.NEWLINE);
    }
    writeDocument(document) {
        const content = this.writePreamble(document) +
            Constants_1.Constants.BEGIN_DOCUMENT +
            this.NEWLINE +
            this.writeHeader(document) +
            Constants_1.Constants.COMMENT +
            this.NEWLINE +
            this.writeTables(document.tables) +
            this.writeAttachments(document.attachments) +
            Constants_1.Constants.END_DOCUMENT +
            this.NEWLINE;
        return content;
    }
    writeHeader(document) {
        switch (document.measuringSystem) {
            case Document_1.MeasuringSystem.METRIC:
                return Constants_1.Constants.Header.METRIC_MEASURE + this.NEWLINE;
            case Document_1.MeasuringSystem.IMPERIAL:
                return Constants_1.Constants.Header.IMPERIAL_MEASURE + this.NEWLINE;
            default:
                throw new Error(`Unknown measuring system ${document.measuringSystem}.`);
        }
    }
    writeTables(tables) {
        return tables.map((x) => this.writeTable(x)).join('');
    }
    writeTable(table) {
        const content = Constants_1.Constants.Table.BEGIN_PREFIX +
            table.name +
            this.NEWLINE +
            Constants_1.Constants.COMMENT +
            this.NEWLINE +
            this.writeColumnDescriptions(table.columnDescriptions) +
            Constants_1.Constants.COMMENT +
            this.NEWLINE +
            this.writeDataSets(table.columnDescriptions, table.dataSets) +
            Constants_1.Constants.COMMENT +
            this.NEWLINE +
            Constants_1.Constants.Table.END_PREFIX +
            table.name +
            this.NEWLINE +
            Constants_1.Constants.COMMENT +
            this.NEWLINE;
        return content;
    }
    writeColumnDescriptions(columDescriptions) {
        const content = Constants_1.Constants.Table.Count.ID +
            Constants_1.Constants.Table.FIELD_SEPARATOR +
            Constants_1.Constants.Table.ColumnDescription.ID +
            Constants_1.Constants.Table.FIELD_SEPARATOR +
            columDescriptions.length +
            this.NEWLINE +
            columDescriptions.columns.map((x) => this.writeColumnDescription(x)).join('');
        return content;
    }
    writeColumnDescription(columnDescription) {
        const content = Constants_1.Constants.Table.ColumnDescription.ID +
            Constants_1.Constants.Table.FIELD_SEPARATOR +
            'AT' +
            Constants_1.Constants.Table.FIELD_SEPARATOR +
            '1' +
            Constants_1.Constants.Table.FIELD_SEPARATOR +
            columnDescription.id.toString().padStart(4) +
            Constants_1.Constants.Table.FIELD_SEPARATOR +
            '1' +
            Constants_1.Constants.Table.FIELD_SEPARATOR +
            '1' +
            Constants_1.Constants.Table.FIELD_SEPARATOR +
            Constants_1.Constants.Table.FIELD_SEPARATOR +
            (Constants_1.Constants.Table.STRING_SEPARATOR + columnDescription.name + Constants_1.Constants.Table.STRING_SEPARATOR).padEnd(40) +
            Constants_1.Constants.Table.FIELD_SEPARATOR +
            Constants_1.Constants.Table.FIELD_SEPARATOR +
            `'${columnDescription.unit ?? ''}'` +
            Constants_1.Constants.Table.FIELD_SEPARATOR +
            this.writeValueType(columnDescription.valueType) +
            this.NEWLINE;
        return content;
    }
    writeValueType(valueType) {
        switch (valueType) {
            case ColumnDescription_1.ValueType.NUMBER:
                return Constants_1.Constants.Table.ColumnDescription.NUMBER_VALUE_TYPE;
            case ColumnDescription_1.ValueType.TEXT:
                return Constants_1.Constants.Table.ColumnDescription.TEXT_VALUE_TYPE;
            case ColumnDescription_1.ValueType.ENUM:
                return Constants_1.Constants.Table.ColumnDescription.ENUM_VALUE_TYPE;
            case ColumnDescription_1.ValueType.BOOLEAN:
                return Constants_1.Constants.Table.ColumnDescription.BOOLEAN_VALUE_TYPE;
            default:
                throw new Error(`Invalid value type ${valueType}.`);
        }
    }
    writeDataSets(columnDescriptions, dataSets) {
        const content = Constants_1.Constants.Table.Count.ID +
            Constants_1.Constants.Table.FIELD_SEPARATOR +
            Constants_1.Constants.Table.Data.ID +
            Constants_1.Constants.Table.FIELD_SEPARATOR +
            dataSets.length +
            this.NEWLINE +
            dataSets.map((x) => this.writeDataSet(columnDescriptions, x)).join('');
        return content;
    }
    writeDataSet(columnDescriptions, dataSet) {
        if (columnDescriptions.length !== dataSet.values.length) {
            throw new Error(`Column description and data set have different counts.`);
        }
        let content = Constants_1.Constants.Table.Data.ID + Constants_1.Constants.Table.FIELD_SEPARATOR;
        let lineLength = Constants_1.Constants.Table.Data.ID.length + 1; // + 1 separator char.
        for (let i = 0; i < dataSet.values.length; ++i) {
            // eslint-disable-next-line security/detect-object-injection
            let output = this.writeValue(columnDescriptions.columns[i].valueType, dataSet.values[i]);
            if (i + 1 < dataSet.values.length) {
                output += Constants_1.Constants.Table.FIELD_SEPARATOR;
            }
            if (lineLength + output.length >= this.MAX_LINE_LENGTH) {
                content += this.NEWLINE + Constants_1.Constants.Table.LINE_CONNECTOR + '  ';
                lineLength = 3; // = line connector + two spaces.
            }
            content += output;
            lineLength += output.length;
        }
        content += this.NEWLINE;
        if (dataSet.attachment?.length) {
            content += Constants_1.Constants.Table.Data.BEGIN_ATTACHMENT + this.NEWLINE + dataSet.attachment.join(this.NEWLINE);
            if (!content.endsWith('\n')) {
                content += this.NEWLINE;
            }
            content += Constants_1.Constants.Table.Data.END_ATTACHMENT + this.NEWLINE;
        }
        return content;
    }
    writeValue(valueType, value) {
        if (value === Constants_1.Constants.Table.EMPTY_FIELD_TOKEN)
            return '';
        switch (valueType) {
            case ColumnDescription_1.ValueType.NUMBER:
            case ColumnDescription_1.ValueType.ENUM:
            case ColumnDescription_1.ValueType.BOOLEAN:
                return value;
            case ColumnDescription_1.ValueType.TEXT:
                return Constants_1.Constants.Table.STRING_SEPARATOR + value + Constants_1.Constants.Table.STRING_SEPARATOR;
            default:
                throw new Error(`Invalid value type ${valueType}.`);
        }
    }
    writeAttachments(attachments) {
        return attachments.map((x) => this.writeAttachment(x)).join('');
    }
    writeAttachment(attachment) {
        const content = Constants_1.Constants.Table.ATTACHMENT_START_PREFIX +
            attachment.name +
            this.NEWLINE +
            attachment.data.join(this.NEWLINE) +
            (attachment.data.length ? this.NEWLINE : '') +
            Constants_1.Constants.Table.ATTACHMENT_STOP_PREFIX +
            attachment.name +
            this.NEWLINE +
            Constants_1.Constants.COMMENT +
            this.NEWLINE;
        return content;
    }
    writePreamble(document) {
        return document.preamble ? document.preamble + this.NEWLINE : '';
    }
}
exports.Writer = Writer;
//# sourceMappingURL=Writer.js.map